##############################################################################
#
# Copyright (c) 2019 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from slapos.grid.promise import PromiseError
from slapos.test.promise.plugin import TestPromisePluginMixin
from slapos.util import str2bytes

import contextlib
from cryptography import x509
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.x509.oid import NameOID
from six.moves import BaseHTTPServer
from base64 import b64encode
import datetime
import ipaddress
import json
import multiprocessing
import os
import six
import socket
import ssl
import tempfile
import time
import unittest

SLAPOS_TEST_IPV4 = os.environ.get('SLAPOS_TEST_IPV4', '127.0.0.1')
SLAPOS_TEST_IPV4_PORT = 57965
HTTPS_ENDPOINT = "https://%s:%s/" % (SLAPOS_TEST_IPV4, SLAPOS_TEST_IPV4_PORT)

# Good and bad username/password for HTTP authentication tests.
TEST_GOOD_USERNAME = 'good username'
TEST_GOOD_PASSWORD = 'good password'
TEST_BAD_USERNAME = 'bad username'
TEST_BAD_PASSWORD = 'bad password'

def createKey():
  key = rsa.generate_private_key(
    public_exponent=65537, key_size=2048, backend=default_backend())
  key_pem = key.private_bytes(
    encoding=serialization.Encoding.PEM,
    format=serialization.PrivateFormat.TraditionalOpenSSL,
    encryption_algorithm=serialization.NoEncryption()
  )
  return key, key_pem


def createCSR(common_name, ip=None):
  key, key_pem = createKey()
  subject_alternative_name_list = []
  if ip is not None:
    subject_alternative_name_list.append(
      x509.IPAddress(ipaddress.ip_address(ip))
    )
  csr = x509.CertificateSigningRequestBuilder().subject_name(x509.Name([
     x509.NameAttribute(NameOID.COMMON_NAME, common_name),
  ]))

  if len(subject_alternative_name_list):
    csr = csr.add_extension(
      x509.SubjectAlternativeName(subject_alternative_name_list),
      critical=False
    )

  csr = csr.sign(key, hashes.SHA256(), default_backend())
  csr_pem = csr.public_bytes(serialization.Encoding.PEM)
  return key, key_pem, csr, csr_pem


class CertificateAuthority(object):
  def __init__(self, common_name):
    self.key, self.key_pem = createKey()
    public_key = self.key.public_key()
    builder = x509.CertificateBuilder()
    builder = builder.subject_name(x509.Name([
      x509.NameAttribute(NameOID.COMMON_NAME, common_name),
    ]))
    builder = builder.issuer_name(x509.Name([
      x509.NameAttribute(NameOID.COMMON_NAME, common_name),
    ]))
    builder = builder.not_valid_before(
      datetime.datetime.utcnow() - datetime.timedelta(days=2))
    builder = builder.not_valid_after(
      datetime.datetime.utcnow() + datetime.timedelta(days=30))
    builder = builder.serial_number(x509.random_serial_number())
    builder = builder.public_key(public_key)
    builder = builder.add_extension(
      x509.BasicConstraints(ca=True, path_length=None), critical=True,
    )
    self.certificate = builder.sign(
      private_key=self.key, algorithm=hashes.SHA256(),
      backend=default_backend()
    )
    self.certificate_pem = self.certificate.public_bytes(
      serialization.Encoding.PEM)

  def signCSR(self, csr):
    builder = x509.CertificateBuilder(
      subject_name=csr.subject,
      extensions=csr.extensions,
      issuer_name=self.certificate.subject,
      not_valid_before=datetime.datetime.utcnow() - datetime.timedelta(days=1),
      not_valid_after=datetime.datetime.utcnow() + datetime.timedelta(days=30),
      serial_number=x509.random_serial_number(),
      public_key=csr.public_key(),
    )
    certificate = builder.sign(
      private_key=self.key,
      algorithm=hashes.SHA256(),
      backend=default_backend()
    )
    return certificate, certificate.public_bytes(serialization.Encoding.PEM)


class TestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
  def do_GET(self):
    """
    Respond to a GET request. You can configure the response as follows:
      - Specify the response code in the URL, e.g. /200
      - Optionally, use an underscore to specify a timeout (in seconds)
        to wait before responding, e.g. /200_5
      - Optionally, use an exclamation mark to require HTTP basic
        authentication, using the credentials TEST_GOOD_USERNAME and
        TEST_GOOD_PASSWORD defined at the top of this file, e.g. /!200
        or /!200_5.
    """
    path = self.path.split('/')[-1]

    if path[0] == '!':
      require_auth = True
      path = path[1:]
    else:
      require_auth = False

    if '_' in path:
      response, timeout = path.split('_')
      response = int(response)
      timeout = int(timeout)
    else:
      timeout = 0
      response = int(path)

    key = b64encode(('%s:%s' % (TEST_GOOD_USERNAME,
                                TEST_GOOD_PASSWORD)).encode()).decode()
    try:
      authorization = self.headers['Authorization']
    except KeyError:
      authorization = None

    time.sleep(timeout)
    if require_auth and authorization != 'Basic ' + key:
      self.send_response(401)
      self.send_header('WWW-Authenticate', 'Basic realm="test"')
      self.end_headers()
      self.wfile.write('bad credentials\n'.encode())
    else:
      self.send_response(response)

      self.send_header("Content-type", "application/json")
      self.end_headers()
      response = {
        'Path': self.path,
      }
      self.wfile.write(str2bytes(json.dumps(response, indent=2)))


class CheckUrlAvailableMixin(TestPromisePluginMixin):
  @classmethod
  def setUpClass(cls):
    cls.another_server_ca = CertificateAuthority(u"Another Server Root CA")
    cls.test_server_ca = CertificateAuthority(u"Test Server Root CA")
    ip = SLAPOS_TEST_IPV4.decode('utf-8') \
         if isinstance(SLAPOS_TEST_IPV4, bytes) \
         else SLAPOS_TEST_IPV4
    key, key_pem, csr, csr_pem = createCSR(
      u"testserver.example.com", ip)
    _, cls.test_server_certificate_pem = cls.test_server_ca.signCSR(csr)

    cls.test_server_certificate_file = tempfile.NamedTemporaryFile(
      delete=False
    )

    cls.test_server_certificate_file.write(
        cls.test_server_certificate_pem + key_pem
      )
    cls.test_server_certificate_file.close()

    cls.test_server_ca_certificate_file = tempfile.NamedTemporaryFile(
      delete=False
    )
    cls.test_server_ca_certificate_file.write(
       cls.test_server_ca.certificate_pem)
    cls.test_server_ca_certificate_file.close()

    def server():
      server = BaseHTTPServer.HTTPServer(
        (SLAPOS_TEST_IPV4, SLAPOS_TEST_IPV4_PORT),
        TestHandler)
      server.socket = ssl.wrap_socket(
        server.socket,
        certfile=cls.test_server_certificate_file.name,
        server_side=True)
      server.serve_forever()

    cls.server_process = multiprocessing.Process(target=server)
    cls.server_process.start()
    for _ in range(20):
      try:
        with contextlib.closing(socket.create_connection((SLAPOS_TEST_IPV4, SLAPOS_TEST_IPV4_PORT))):
          break
      except Exception:
        time.sleep(.1)

  @classmethod
  def tearDownClass(cls):
    cls.server_process.terminate()
    cls.server_process.join()
    for p in [
      cls.test_server_certificate_file.name,
      cls.test_server_ca_certificate_file.name,
    ]:
      try:
        os.unlink(p)
      except Exception:
        pass

  def setUp(self):
    TestPromisePluginMixin.setUp(self)
    self.promise_name = "check-url-available.py"

    self.success_template = \
      ("non-authenticated request to %r succeeded "
       "(returned expected code %d)")
    self.authenticated_success_template = \
      ("authenticated request to %r succeeded "
       "(returned expected code %d)")
    self.ignored_success_template = \
      ("non-authenticated request to %r succeeded "
       "(return code ignored)")
    self.authenticated_ignored_success_template = \
      ("authenticated request to %r succeeded "
       "(return code ignored)")

  def make_content(self, option_dict):
    content = """from slapos.promise.plugin.check_url_available import RunPromise

extra_config_dict = {
"""
    for option in option_dict.items():
      content += "\n  '%s': %r," % option

    return content + "\n}"

  def tearDown(self):
    TestPromisePluginMixin.tearDown(self)


class TestCheckUrlAvailable(CheckUrlAvailableMixin):

  def test_check_url_bad(self):
    content = self.make_content({
      'url': 'https://',
      'timeout': 10,
      'ignore-code': 0,
    })
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(
      result['result']['message'],
      "ERROR: Invalid URL %s'https://': No host supplied" %
        ('' if six.PY3 else 'u')
    )

  def test_check_url_malformed(self):
    content = self.make_content({
      'url': '',
      'timeout': 10,
      'ignore-code': 0,
    })
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(
      result['result']['message'],
      "ERROR: Invalid URL '': No schema supplied. Perhaps you meant http://?"
    )

  def test_check_url_site_off(self):
    content = self.make_content({
      'url': 'https://localhost:56789/site',
      'timeout': 10,
      'ignore-code': 0,
    })
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(
      result['result']['message'],
      "ERROR connection not possible while accessing "
      "'https://localhost:56789/site'"
    )

  def test_check_200(self):
    url = HTTPS_ENDPOINT + '200'
    content = self.make_content({
      'url': url,
      'timeout': 10,
      'ignore-code': 0,
    })
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(
      result['result']['message'],
      self.success_template % (url, 200)
    )

  def test_check_200_verify(self):
    url = HTTPS_ENDPOINT + '200'
    content = self.make_content({
      'url': url,
      'timeout': 10,
      'ignore-code': 0,
      'verify': 1,
    })
    try:
      old = os.environ.get('REQUESTS_CA_BUNDLE')
      # simulate system provided CA bundle
      os.environ[
        'REQUESTS_CA_BUNDLE'] = self.test_server_ca_certificate_file.name
      self.writePromise(self.promise_name, content)
      self.configureLauncher()
      self.launcher.run()
    finally:
      if old is None:
        del os.environ['REQUESTS_CA_BUNDLE']
      else:
        os.environ['REQUESTS_CA_BUNDLE'] = old

    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(
      result['result']['message'],
      self.success_template % (url, 200)
    )

  def test_check_200_verify_fail(self):
    url = HTTPS_ENDPOINT + '200'
    content = self.make_content({
      'url': url,
      'timeout': 10,
      'ignore-code': 0,
      'verify': 1,
    })
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(
      result['result']['message'],
      "ERROR SSL verify failed while accessing %r" % (url,)
    )

  def test_check_200_verify_own(self):
    url = HTTPS_ENDPOINT + '200'
    content = self.make_content({
      'url': url,
      'timeout': 10,
      'ignore-code': 0,
      'ca-cert-file': self.test_server_ca_certificate_file.name
    })
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(
      result['result']['message'],
      self.success_template % (url, 200)
    )

  def test_check_401(self):
    url = HTTPS_ENDPOINT + '401'
    content = self.make_content({
      'url': url,
      'timeout': 10,
      'ignore-code': 0,
    })
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(
      result['result']['message'],
      ("non-authenticated request to %r failed "
       "(returned 401, expected 200)") % (url,)
    )

  def test_check_401_ignore_code(self):
    url = HTTPS_ENDPOINT + '401'
    content = self.make_content({
      'url': url,
      'timeout': 10,
      'ignore-code': 1,
    })
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(
      result['result']['message'],
      self.ignored_success_template % (url,)
    )

  def test_check_512_http_code(self):
    url = HTTPS_ENDPOINT + '512'
    content = self.make_content({
      'url': url,
      'timeout': 10,
      'ignore-code': 0,
      'http-code': 512,
    })
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(
      result['result']['message'],
      self.success_template % (url, 512)
    )

  # Test bad HTTP code.
  def test_check_bad_http_code(self):
    url = HTTPS_ENDPOINT + '412'
    content = self.make_content({
      'url': url,
      'timeout': 10,
      'ignore-code': 0,
      'http-code': 732,
    })
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(
      result['result']['message'],
      ("non-authenticated request to %r failed "
       "(returned 412, expected 732)") % (url,)
    )

  # Test normal authentication success.
  def test_check_authenticate_success(self):
    url = HTTPS_ENDPOINT + '!200'
    content = self.make_content({
      'url': url,
      'username': TEST_GOOD_USERNAME,
      'password': TEST_GOOD_PASSWORD,
    })
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(
      result['result']['message'],
      self.authenticated_success_template % (url, 200)
    )

  # Test that supplying a username/password still succeeds when the
  # server doesn't require them.
  def test_check_authenticate_no_password_needed(self):
    url = HTTPS_ENDPOINT + '200'
    content = self.make_content({
      'url': url,
      'username': TEST_GOOD_USERNAME,
      'password': TEST_GOOD_PASSWORD,
    })
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(
      result['result']['message'],
      self.authenticated_success_template % (url, 200)
    )

  # Test authentication failure due to bad password.
  def test_check_authenticate_bad_password(self):
    url = HTTPS_ENDPOINT + '!200'
    content = self.make_content({
      'url': url,
      'username': TEST_BAD_USERNAME,
      'password': TEST_BAD_PASSWORD,
    })
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(
      result['result']['message'],
      ("authenticated request to %r failed "
       "(returned 401, expected 200)") % (url,)
    )

  # Test authentication failure due to no password being given to a
  # protected server.
  def test_check_authenticate_no_password_given(self):
    url = HTTPS_ENDPOINT + '!200'
    content = self.make_content({
      'url': url,
    })
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(
      result['result']['message'],
      ("non-authenticated request to %r failed "
       "(returned 401, expected 200)") % (url,)
    )

  # Test that authentication and HTTP code can be used together.
  def test_check_authenticate_http_code(self):
    url = HTTPS_ENDPOINT + '!412'
    content = self.make_content({
      'url': url,
      'username': TEST_GOOD_USERNAME,
      'password': TEST_GOOD_PASSWORD,
      'http-code': 412
    })
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(
      result['result']['message'],
      self.authenticated_success_template % (url, 412)
    )

  # Test that authentication and igore-code can be used together.
  def test_check_authenticate_ignore_code(self):
    url = HTTPS_ENDPOINT + '!404'
    content = self.make_content({
      'url': url,
      'username': TEST_GOOD_USERNAME,
      'password': TEST_GOOD_PASSWORD,
      'ignore-code': 1
    })
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(
      result['result']['message'],
      self.authenticated_ignored_success_template % (url,)
    )

class TestCheckUrlAvailableTimeout(CheckUrlAvailableMixin):
  def test_check_200_timeout(self):
    url = HTTPS_ENDPOINT + '200_5'
    content = self.make_content({
      'url': url,
      'timeout': 1,
      'ignore-code': 0,
    })
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(
      result['result']['message'],
      "Error: Promise timed out after 0.5 seconds",
    )


if __name__ == '__main__':
  unittest.main()



##############################################################################
#
# Copyright (c) 2015 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import unittest
import os.path
from slapos.networkbench import dnsbench
from slapos.networkbench.ping import ping, ping6
from slapos.networkbench.http import request


DNS_EXPECTED_LIST = ["161.97.166.226", "176.31.129.213"]

class TestDNSBench(unittest.TestCase):

  def test_dnsbench_ok(self):
    """
    Test dns resolution
    """
    info = dnsbench.resolve(
       "eu.web.vifib.com", DNS_EXPECTED_LIST)
  
    self.assertEqual(info[0], 'DNS')
    self.assertEqual(info[1], 'eu.web.vifib.com')
    self.assertEqual(info[2], 200)

    self.assertLess(info[3], 30)
    self.assertEqual(info[4], 'OK', 
          "%s != OK, full info: %s" % (info[4], info) )
    self.assertEqual(set(info[5]), set([u'161.97.166.226', u'176.31.129.213']),
          "%s != set([u'161.97.166.226', u'176.31.129.213']), full info: %s" % (set(info[5]), info))
    
  def test_dnsbench_fail(self):
    """ Test dns failure resolution
    """
    info = dnsbench.resolve(
       "thisdomaindontexist.erp5.com")
   
    self.assertEqual(info[0], 'DNS')
    self.assertEqual(info[1], 'thisdomaindontexist.erp5.com')
    self.assertEqual(info[2], 600)

    self.assertLess(info[3], 30)
    self.assertEqual(info[4], 'Cannot resolve the hostname')
    self.assertEqual(info[5], [])

  def test_dnsbench_unexpected(self):
    """ Test dns unexpected resolution
    """
    info = dnsbench.resolve(
       "www.erp5.com", [DNS_EXPECTED_LIST[0]])
   
    self.assertEqual(info[0], 'DNS')
    self.assertEqual(info[1], 'www.erp5.com')
    self.assertEqual(info[2], 200)

    self.assertLess(info[3], 30)
    self.assertEqual(info[4], 'UNEXPECTED')
    self.assertTrue(info[5].startswith("['161.97.166.226'] (expected) != "))

class TestPing(unittest.TestCase):

  def test_ping_ok(self):
    info = ping("localhost")
    self.assertEqual(info[0], 'PING')
    self.assertEqual(info[1], 'localhost')
    self.assertEqual(info[2], 200)
    self.assertLess(float(info[3]), 0.2)
    self.assertEqual(info[4], '0')
    self.assertTrue(info[5].startswith("min"))

  def test_ping_fail(self):
    info = ping("couscous")
    self.assertEqual(info[0], 'PING')
    self.assertEqual(info[1], 'couscous')
    self.assertEqual(info[2], 600)
    self.assertEqual(info[3], 'failed')
    self.assertEqual(info[4], -1)
    self.assertEqual(info[5], 'Fail to parser ping output')


  def test_ping6_ok(self):
    info = ping6("localhost")
    self.assertEqual(info[0], 'PING6')
    self.assertEqual(info[1], 'localhost')
    self.assertEqual(info[2], 200)
    self.assertLess(float(info[3]), 0.2)
    self.assertEqual(info[4], '0')
    self.assertTrue(info[5].startswith("min"))

  def test_ping6_fail(self):
    info = ping6("couscous")
    self.assertEqual(info[0], 'PING6')
    self.assertEqual(info[1], 'couscous')
    self.assertEqual(info[2], 600)
    self.assertEqual(info[3], 'failed')
    self.assertEqual(info[4], -1)
    self.assertEqual(info[5], 'Fail to parser ping output')


class TestHTTPBench(unittest.TestCase):

  def test_request_ok(self):
    """ This test is way to badly written as it depends on
        www.erp5.com for now, please replace it
    """
    info = request("https://www.erp5.com", {})

    self.assertEqual(info[0], 'GET')
    self.assertEqual(info[1], 'https://www.erp5.com')
    self.assertEqual(info[2], 200)
    self.assertEqual(len(info[3].split(';')), 5 )
    self.assertEqual(info[4], "OK")

  def test_request_expected_response(self):
    """ This test is way to badly written as it depends on
        www.erp5.com for now, please replace it
    """
    info = request("https://www.erp5.com", {"expected_response": 200})

    self.assertEqual(info[0], 'GET')
    self.assertEqual(info[1], 'https://www.erp5.com')
    self.assertEqual(info[2], 200)
    self.assertEqual(len(info[3].split(';')), 5 )
    self.assertEqual(info[4], "OK")

  def test_request_expected_redirection(self):
    """ This test is way to badly written as it depends on
        www.erp5.com for now, please replace it
    """
    info = request("http://www.erp5.com", {"expected_response": 302})

    self.assertEqual(info[0], 'GET')
    self.assertEqual(info[1], 'http://www.erp5.com')
    self.assertEqual(info[2], 302)
    self.assertEqual(len(info[3].split(';')), 5 )
    self.assertEqual(info[4], "OK")


  def test_request_expected_text(self):
    """ This test is way to badly written as it depends on
        www.erp5.com for now, please replace it
    """
    info = request("https://www.erp5.com", {"expected_text": "ERP5"})

    self.assertEqual(info[0], 'GET')
    self.assertEqual(info[1], 'https://www.erp5.com')
    self.assertEqual(info[2], 200)
    self.assertEqual(len(info[3].split(';')), 5 )
    self.assertEqual(info[4], "OK")


  def test_request_fail(self):
    """ Test unreachable URL
    """
    info = request("http://thisurldontexist.erp5.com", {})

    self.assertEqual(info[0], 'GET')
    self.assertEqual(info[1], 'http://thisurldontexist.erp5.com')
    self.assertEqual(info[2], 0)
    self.assertEqual(len(info[3].split(';')), 5 )
    self.assertEqual(info[4], "FAIL")

  def test_request_unexpected_response(self):
    """ This test is way to badly written as it depends on
        www.erp5.com for now, please replace it
    """
    info = request("http://www.erp5.com", {"expected_response": 200})

    self.assertEqual(info[0], 'GET')
    self.assertEqual(info[1], 'http://www.erp5.com')
    self.assertEqual(info[2], 302)
    self.assertEqual(len(info[3].split(';')), 5 )
    self.assertEqual(info[4], "UNEXPECTED (200 != 302)")

  def test_request_unexpected_text(self):
    """ This test is way to badly written as it depends on
        www.erp5.com for now, please replace it.
    """
    info = request("https://www.erp5.com", {"expected_text": "COUSCOUS"})

    self.assertEqual(info[0], 'GET')
    self.assertEqual(info[1], 'https://www.erp5.com')
    self.assertEqual(info[2], 200)
    self.assertEqual(len(info[3].split(';')), 5 )
    self.assertEqual(info[4], "UNEXPECTED (COUSCOUS not in page content)")



##############################################################################
#
# Copyright (c) 2019 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from slapos.grid.promise import PromiseError
from slapos.test.promise.plugin import TestPromisePluginMixin
from slapos.grid.svcbackend import getSupervisorRPC

import tempfile
import os
import unittest
import shutil
import psutil
import textwrap
import subprocess


class TestCheckServiceState(TestPromisePluginMixin):

  def setUp(self):
    TestPromisePluginMixin.setUp(self)
    self._tempdir = tempfile.mkdtemp()
    self._supervisor_socket = os.path.join(self._tempdir, 'sv.sock')
    self._supervisord_log = os.path.join(self._tempdir, 'supervisord.log')
    self._supervisord_pid = os.path.join(self._tempdir, 'supervisord.pid')
    self._supervisor_config = os.path.join(self._tempdir, 'supervisor.conf')
    self._service_name = 'test-service'
    self._test_log = os.path.join(self._tempdir, '%s.log' % (self._service_name,))

    with open(self._supervisor_config, 'w') as f:
      f.write(textwrap.dedent(
        """
        [unix_http_server]
        file = {self._supervisor_socket}

        [supervisorctl]
        serverurl = unix://{self._supervisor_socket}

        [supervisord]
        logfile = {self._supervisord_log}
        pidfile = {self._supervisord_pid}

        [rpcinterface:supervisor]
        supervisor.rpcinterface_factory = supervisor.rpcinterface:make_main_rpcinterface

        [program:{self._service_name}]
        command = sleep 10000
        autostart = false
        autorestart = false
        startretries = 0
        startsecs = 0
        redirect_stderr = true
        stdout_logfile = {self._test_log}

        """).format(**locals()))

    subprocess.check_output(
        ['supervisord', '--configuration', self._supervisor_config],
        cwd=self._tempdir,
    )

    self.promise_name = 'check-service-state.py'
    self.base_content = textwrap.dedent(
      """
      from slapos.promise.plugin.check_service_state import RunPromise
      extra_config_dict = {
        'run-directory': %(rundir)r,
        'service': %(service)r,
        'expect': %%(expect)r,
      }
      """) % {'rundir': self._tempdir, 'service': self._service_name}

  def tearDown(self):
    TestPromisePluginMixin.tearDown(self)
    with getSupervisorRPC(self._supervisor_socket) as supervisor:
      supervisor_process = psutil.Process(supervisor.getPID())
      supervisor.stopAllProcesses()
      supervisor.shutdown()
    _, alive = psutil.wait_procs([supervisor_process], timeout=300)
    if alive:
      supervisor_process.kill()
    shutil.rmtree(self._tempdir)

  def test_running_expect_running(self):
    with getSupervisorRPC(self._supervisor_socket) as supervisor:
      supervisor.startProcess(self._service_name)
      assert supervisor.getProcessInfo(self._service_name)['statename'] == 'RUNNING'
    content = self.base_content % {'expect': 'running'}
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(
      result['result']['message'],
      "OK service %r is in expected state 'running'" % (self._service_name,)
    )

  def test_running_expect_stopped(self):
    with getSupervisorRPC(self._supervisor_socket) as supervisor:
      supervisor.startProcess(self._service_name)
      assert supervisor.getProcessInfo(self._service_name)['statename'] == 'RUNNING'
    content = self.base_content % {'expect': 'stopped'}
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(
      result['result']['message'],
      "ERROR service %r is in state 'running' (expected 'stopped')" % (self._service_name,)
    )

  def test_running_expect_undefined(self):
    with getSupervisorRPC(self._supervisor_socket) as supervisor:
      supervisor.startProcess(self._service_name)
      assert supervisor.getProcessInfo(self._service_name)['statename'] == 'RUNNING'
    content = self.base_content % {'expect': 'undefined'}
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(
      result['result']['message'],
      "OK service %r is allowed to be in any state (expect = 'undefined')" % (self._service_name,)
    )

  def test_stopped_expect_running(self):
    with getSupervisorRPC(self._supervisor_socket) as supervisor:
      assert supervisor.getProcessInfo(self._service_name)['statename'] == 'STOPPED'
    content = self.base_content % {'expect': 'running'}
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(
      result['result']['message'],
      "ERROR service %r is in state 'stopped' (expected 'running')" % (self._service_name,)
    )

  def test_stopped_expect_stopped(self):
    with getSupervisorRPC(self._supervisor_socket) as supervisor:
      assert supervisor.getProcessInfo(self._service_name)['statename'] == 'STOPPED'
    content = self.base_content % {'expect': 'stopped'}
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(
      result['result']['message'],
      "OK service %r is in expected state 'stopped'" % (self._service_name,)
    )

  def test_stopped_expect_undefined(self):
    with getSupervisorRPC(self._supervisor_socket) as supervisor:
      assert supervisor.getProcessInfo(self._service_name)['statename'] == 'STOPPED'
    content = self.base_content % {'expect': 'undefined'}
    self.writePromise(self.promise_name, content)
    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(
      result['result']['message'],
      "OK service %r is allowed to be in any state (expect = 'undefined')" % (self._service_name,)
    )

if __name__ == '__main__':
  unittest.main()



##############################################################################
#
# Copyright (c) 2018 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from slapos.test.promise.plugin import TestPromisePluginMixin
from slapos.grid.promise import PromiseError
import os

class TestPartitionDeploymentState(TestPromisePluginMixin):

  def setUp(self):
    TestPromisePluginMixin.setUp(self)
    log_folder = os.path.join(self.partition_dir, 'var/log')
    os.makedirs(log_folder)

  def writeState(self, message):
    path = os.path.join(self.partition_dir, '.slapgrid-%s-error.log' % self.partition_id)
    with open(path, 'w') as f:
      f.write(message)

  def test_partition_deployment_state_ok(self):
    content = """
from slapos.promise.plugin.check_partition_deployment_state import RunPromise
"""
    self.writePromise("partition-deployment-state.py", content)
    self.configureLauncher()
    self.launcher.run()
    result = self.getPromiseResult("partition-deployment-state.py")
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(result['result']['message'], 'buildout is OK')

  def test_partition_deployment_state_failed(self):
    content = """
from slapos.promise.plugin.check_partition_deployment_state import RunPromise
"""
    message = "Slapgrid failed to process partition XXXXXXX"
    self.writePromise("partition-deployment-state.py", content)
    self.writeState(message)
    self.configureLauncher()
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult("partition-deployment-state.py")
    buildout_output = os.path.join(self.partition_dir, 'var/log', 'slapgrid-%s-error.log' % self.partition_id)

    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(result['result']['message'], 'Buildout failed to process %s.' % self.partition_id)
    with open(buildout_output) as f:
      self.assertEqual(f.read(), message)

if __name__ == '__main__':
  unittest.main()



from zope.interface import implementer
from slapos.grid.promise import interface
from slapos.grid.promise.generic import GenericPromise
try:
  import subprocess32 as subprocess
except ImportError:
  import subprocess

@implementer(interface.IPromise)
class RunPromise(GenericPromise):
  def __init__(self, config):
    super(RunPromise, self).__init__(config)
    # check configuration every 5 minutes (only for anomaly)
    self.setPeriodicity(minute=int(self.getConfig('frequency', 5)))

  def sense(self):
    """
      Run frontend validatation script
    """

    validate_script = self.getConfig('verification-script')
    if not validate_script:
      raise ValueError("'verification-script' was not set in promise parameters.")
    try:
      subprocess.check_output(validate_script, stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as e:
      message = e.output
      self.logger.error(message if str is bytes else
                        message.decode('utf-8', 'replace'))
    else:
      self.logger.info("OK")

  def anomaly(self):
    return self._anomaly(result_count=1, failure_amount=1)



from setuptools import setup, find_packages
import glob
import os

version = '0.124'
name = 'slapos.toolbox'
long_description = open("README.rst").read() + "\n"

for f in sorted(glob.glob(os.path.join('slapos', 'README.*.rst'))):
  long_description += '\n' + open(f).read() + '\n'

long_description += open("CHANGES.txt").read() + "\n"

test_require = ['mock', 'cryptography',]

setup(name=name,
      version=version,
      description="SlapOS toolbox.",
      long_description=long_description,
      classifiers=[
          "Programming Language :: Python",
        ],
      keywords='slapos toolbox',
      license='GPLv3',
      namespace_packages=['slapos'],
      packages=find_packages(),
      include_package_data=True,
      maintainer="Nexedi",
      maintainer_email="info@nexedi.com",
      url="https://lab.nexedi.com/nexedi/slapos.toolbox",
      install_requires=(
        'Flask', # needed by servers
        'atomize', # needed by pubsub
        'feedparser', # needed by pubsub
        'lockfile', # used by equeue
        'lxml', # needed for xml parsing
        'psutil', # needed for playing with processes in portable way
        'setuptools', # namespaces
        'slapos.core', # as it provides library for slap
        'xml_marshaller', # needed to dump information
        'GitPython', #needed for git manipulation into slaprunner
        'croniter', # needed to know cron schedule
        'pytz', # needed to manipulate timezone
        'tzlocal', # needed to manipulate timezone
        'backports.lzma',
        'passlib',
        'netifaces',
        'erp5.util',
        'PyRSS2Gen',
        'dnspython',
        'requests',
        'jsonschema',
        'zc.buildout',
        'pycurl',
        'six',
        'cryptography',
        'click',
        'ipaddress; python_version<"3"',
      ),
      extras_require = {
        'lampconfigure':  ["mysqlclient"], #needed for MySQL Database access
        'zodbpack': ['ZODB3'], # needed to play with ZODB
        'flask_auth' : ["Flask-Auth"],
        'test': test_require,
      },
      tests_require=test_require,
      zip_safe=False, # proxy depends on Flask, which has issues with
                      # accessing templates
      entry_points={
        'console_scripts': [
          'agent = slapos.agent.agent:main',
          'apache-mpm-watchdog = slapos.promise.apache_mpm_watchdog:main',
          'check-computer-memory = slapos.promise.check_computer_memory:main',
          'check-web-page-http-cache-hit = slapos.promise.check_web_page_http_cache_hit:main',
          'check-feed-as-promise = slapos.checkfeedaspromise:main',
          'check-apachedex-result = slapos.promise.check_apachedex_result:main',
          'check-slow-queries-digest-result = slapos.promise.check_slow_queries_digest_result:main',
          'equeue = slapos.equeue:main',
          'generatefeed = slapos.generatefeed:main',
          'htpasswd = slapos.htpasswd:main',
          'is-local-tcp-port-opened = slapos.promise.is_local_tcp_port_opened:main',
          'is-process-older-than-dependency-set = slapos.promise.is_process_older_than_dependency_set:main',
          'killpidfromfile = slapos.systool:killpidfromfile', # BBB
          'monitor.bootstrap = slapos.monitor.monitor:main',
          'monitor.collect = slapos.monitor.collect:main',
          'monitor.statistic = slapos.monitor.build_statistic:main',
          'monitor.runpromise = slapos.monitor.runpromise:main',
          'monitor.genstatus = slapos.monitor.globalstate:main',
          'monitor.configwrite = slapos.monitor.monitor_config_write:main',
          'runResiliencyUnitTestTestNode = slapos.resiliencytest:runUnitTest',
          'runResiliencyScalabilityTestNode = slapos.resiliencytest:runResiliencyTest',
          'runApacheDex = slapos.apachedex:main',
          'lampconfigure = slapos.lamp:run [lampconfigure]',
          'onetimedownload = slapos.onetimedownload:main',
          'onetimeupload = slapos.onetimeupload:main',
          'pubsubnotifier = slapos.pubsub.notifier:main',
          'pubsubserver = slapos.pubsub:main',
          'qemu-qmp-client = slapos.qemuqmpclient:main',
          'rdiffbackup.genstatrss = slapos.resilient.rdiffBackupStat2RSS:main',
          'runner-exporter = slapos.resilient.runner_exporter:runExport',
          'runner-importer-post-notification-run = slapos.resilient.runner_importer:postNotificationRun',
          'backup-identity-script-excluding-path = slapos.resilient.identity_script_excluding_path:calculateSignature',
          'securedelete = slapos.securedelete:main',
          'slapos-kill = slapos.systool:kill',
          'slaprunnertest = slapos.runner.runnertest:main',
          'slaprunnerteststandalone = slapos.runner.runnertest:runStandaloneUnitTest',
          'zodbpack = slapos.zodbpack:run [zodbpack]',
          'networkbench = slapos.networkbench:main',
          'cachechecker = slapos.cachechecker:web_checker_utility',
          'dnsresolver = slapos.dnsresolver:cli',
        ]
      },
      test_suite='slapos.test',
    )



##############################################################################
#
# Copyright (c) 2018 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from slapos.test.promise.plugin import TestPromisePluginMixin
from slapos.grid.promise import PromiseError
import os
from datetime import datetime,timedelta

class TestBackupserverCheckBackup(TestPromisePluginMixin):

  def setUp(self):
    TestPromisePluginMixin.setUp(self)
    log_folder = os.path.join(self.partition_dir, 'var/log')
    os.makedirs(log_folder)

    self.status_name = 'SOFTINSTTEST_status.txt'
    self.status_fullpath = os.path.join(log_folder, self.status_name)
    self.cron_frequency = '0 0 * * *'

    self.promise_name = "backupserver_check_backup.py"

    content = """from slapos.promise.plugin.backupserver_check_backup import RunPromise


extra_config_dict = {{
  'cron_frequency': '{}',
  'statistic_dirbasename': 'DUMMY_STATISTIC_DIRBASENAME',
  'statistic_name': 'DUMMY_STATISTIC_NAME',
  'status_dirbasename': 'DUMMY_STATUS_DIRBASENAME',
  'status_fullpath': '{}',
  'status_name': '{}',
  'monitor_url': 'https://DUMMY_MONITOR_URL',
}}
""".format(self.cron_frequency, self.status_fullpath, self.status_name)
    self.writePromise(self.promise_name, content)

  def tearDown(self):
    TestPromisePluginMixin.tearDown(self)
    if os.path.exists(self.status_fullpath):
      os.remove(self.status_fullpath)

  def format_status(self, date, status):
    """" return a string formatted like backupserver status """
    if date.tzinfo is not None:
      raise "Date should be UTC"
    return "{}+0000, DUMMY_STATISTIC_NAME, SOFTINSTTEST, backup {}\n".format(date.replace(microsecond=0).isoformat(), status)

  def test_check_backup_ok(self):
    now = datetime.utcnow()
    with open(self.status_fullpath, 'w') as f:
      f.write(self.format_status(now, "running"))
      f.write(self.format_status(now, "success"))

    self.configureLauncher(enable_anomaly=True)
    self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], False)
    self.assertIn("Backup OK", result['result']['message'])

  def test_check_backup_fail(self):
    now = datetime.utcnow()
    with open(self.status_fullpath, 'w') as f:
      f.write(self.format_status(now, "running"))
      f.write(self.format_status(now, "failed"))

    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertIn("Backup FAILED", result['result']['message'])

  def test_check_backup_too_long(self):
    now = datetime.utcnow()
    long_ago = now - timedelta(days = 2)
    with open(self.status_fullpath, 'w') as f:
      f.write(self.format_status(long_ago, "running"))

    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    result = self.getPromiseResult(self.promise_name)
    self.assertEqual(result['result']['failed'], True)
    self.assertIn("Backup didn't start at correct time", result['result']['message'])

if __name__ == '__main__':
  unittest.main()



from slapos.grid.promise import PromiseError
from slapos.test.promise.plugin import TestPromisePluginMixin

import os
import shutil
import tempfile


class CheckSurykatkaJSONMixin(TestPromisePluginMixin):
  promise_name = 'check-surykatka-json.py'

  def setUp(self):
    self.working_directory = tempfile.mkdtemp()
    self.json_file = os.path.join(self.working_directory, 'surykatka.json')
    self.addCleanup(shutil.rmtree, self.working_directory)
    TestPromisePluginMixin.setUp(self)

  def writeSurykatkaPromise(self, d=None):
    if d is None:
      d = {}
    content_list = [
      "from slapos.promise.plugin.check_surykatka_json import RunPromise"]
    content_list.append('extra_config_dict = {')
    for k, v in d.items():
      content_list.append("  '%s': '%s'," % (k, v))
    content_list.append('}')
    self.writePromise(self.promise_name, '\n'.join(content_list))

  def writeSurykatkaJson(self, content):
    with open(self.json_file, 'w') as fh:
      fh.write(content)

  def assertFailedMessage(self, result, message):
    self.assertEqual(result['result']['failed'], True)
    self.assertEqual(
      result['result']['message'],
      message)

  def assertPassedMessage(self, result, message):
    self.assertEqual(result['result']['failed'], False)
    self.assertEqual(
      result['result']['message'],
      message)


class TestCheckSurykatkaJSON(CheckSurykatkaJSONMixin):
  def test_no_config(self):
    self.writeSurykatkaPromise()
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "ERROR File '' does not exists")

  def test_not_existing_file(self):
    self.writeSurykatkaPromise({'json-file': self.json_file})
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "ERROR File '%s' does not exists" % (self.json_file,))

  def test_empty_file(self):
    self.writeSurykatkaPromise({'json-file': self.json_file})
    self.writeSurykatkaJson('')
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "ERROR loading JSON from '%s'" % (self.json_file,))


class TestCheckSurykatkaJSONUnknownReport(CheckSurykatkaJSONMixin):
  def test(self):
    self.writeSurykatkaPromise(
      {
        'report': 'NOT_EXISTING_ENTRY',
        'json-file': self.json_file,
      }
    )
    self.writeSurykatkaJson("""{
}
""")
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "ERROR Report 'NOT_EXISTING_ENTRY' is not supported")


class TestCheckSurykatkaJSONBotStatus(CheckSurykatkaJSONMixin):
  def test(self):
    self.writeSurykatkaPromise(
      {
        'report': 'bot_status',
        'json-file': self.json_file,
        'test-utcnow': 'Wed, 13 Dec 2222 09:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "bot_status": [
        {
            "date": "Wed, 13 Dec 2222 09:10:11 -0000",
            "text": "loop"
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    self.launcher.run()
    self.assertPassedMessage(
      self.getPromiseResult(self.promise_name),
      "bot_status: OK Last bot status"
    )

  def test_no_loop(self):
    self.writeSurykatkaPromise(
      {
        'report': 'bot_status',
        'json-file': self.json_file,
        'test-utcnow': 'Wed, 13 Dec 2222 09:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "bot_status": [
        {
            "date": "Wed, 13 Dec 2222 09:10:11 -0000",
            "text": "error"
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "bot_status: ERROR bot_status is 'error' instead of 'loop' in '%s'" % (
        self.json_file,)
    )

  def test_bot_status_future(self):
    self.writeSurykatkaPromise(
      {
        'report': 'bot_status',
        'json-file': self.json_file,
        'test-utcnow': 'Wed, 13 Dec 2222 09:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "bot_status": [
        {
            "date": "Wed, 13 Dec 2223 09:10:11 -0000",
            "text": "loop"
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "bot_status: ERROR Last bot datetime is in future"
    )

  def test_bot_status_old(self):
    self.writeSurykatkaPromise(
      {
        'report': 'bot_status',
        'json-file': self.json_file,
        'test-utcnow': 'Wed, 13 Dec 2223 09:26:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "bot_status": [
        {
            "date": "Wed, 13 Dec 2223 09:10:11 -0000",
            "text": "loop"
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "bot_status: ERROR Last bot datetime is more than 15 minutes old"
    )

  def test_not_bot_status(self):
    self.writeSurykatkaPromise(
      {
        'report': 'bot_status',
        'json-file': self.json_file,
      }
    )
    self.writeSurykatkaJson("""{
}
""")
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "bot_status: ERROR 'bot_status' not in '%s'" % (self.json_file,))

  def test_empty_bot_status(self):
    self.writeSurykatkaPromise(
      {
        'report': 'bot_status',
        'json-file': self.json_file,
      }
    )
    self.writeSurykatkaJson("""{
  "bot_status": []
}
""")
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "bot_status: ERROR 'bot_status' empty in '%s'" % (self.json_file,))


class TestCheckSurykatkaJSONHttpQuery(CheckSurykatkaJSONMixin):
  def test(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'https://www.erp5.com/',
        'status-code': '302',
        'ip-list': '127.0.0.1 127.0.0.2',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.1",
            "status_code": 302,
            "url": "https://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.2",
            "status_code": 302,
            "url": "https://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "176.31.129.213",
            "status_code": 200,
            "url": "https://www.erp5.org/"
        }
    ],
    "ssl_certificate": [
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.1",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        },
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.2",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    self.launcher.run()
    self.assertPassedMessage(
      self.getPromiseResult(self.promise_name),
      "https://www.erp5.com/ : http_query: OK with status code 302 on IPs "
      "127.0.0.1 127.0.0.2 ssl_certificate: OK Certificate on 127.0.0.1 will "
      "expire on Mon, 13 Jul 2020 12:00:00 -0000, which is more than 15 days"
    )

  def test_maximum_elapsed_time(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'https://www.erp5.com/',
        'status-code': '302',
        'ip-list': '127.0.0.1 127.0.0.2',
        'maximum-elapsed-time': '5',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.1",
            "status_code": 302,
            "url": "https://www.erp5.com/",
            "total_seconds": 4
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.2",
            "status_code": 302,
            "url": "https://www.erp5.com/",
            "total_seconds": 4
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "176.31.129.213",
            "status_code": 200,
            "url": "https://www.erp5.org/",
            "total_seconds": 4
        }
    ],
    "ssl_certificate": [
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.1",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        },
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.2",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    self.launcher.run()
    self.assertPassedMessage(
      self.getPromiseResult(self.promise_name),
      "https://www.erp5.com/ : http_query: OK with status code 302 on IPs "
      "127.0.0.1 127.0.0.2 ssl_certificate: OK Certificate on 127.0.0.1 will "
      "expire on Mon, 13 Jul 2020 12:00:00 -0000, which is more than 15 days "
      "elapsed_time: OK IP 127.0.0.1 replied in less time than maximum 5.00s "
      "elapsed_time: OK IP 127.0.0.2 replied in less time than maximum 5.00s"
    )

  def test_maximum_elapsed_time_too_long(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'https://www.erp5.com/',
        'status-code': '302',
        'ip-list': '127.0.0.1 127.0.0.2',
        'maximum-elapsed-time': '5',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.1",
            "status_code": 302,
            "url": "https://www.erp5.com/",
            "total_seconds": 6
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.2",
            "status_code": 302,
            "url": "https://www.erp5.com/",
            "total_seconds": 0
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "176.31.129.213",
            "status_code": 200,
            "url": "https://www.erp5.org/",
            "total_seconds": 4
        }
    ],
    "ssl_certificate": [
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.1",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        },
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.2",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "https://www.erp5.com/ : elapsed_time: ERROR IP 127.0.0.1 replied in "
      "more time than maximum 5.00s elapsed_time: ERROR IP 127.0.0.2 failed "
      "to reply http_query: OK with status code 302 on IPs 127.0.0.1 "
      "127.0.0.2 ssl_certificate: OK Certificate on 127.0.0.1 will expire on "
      "Mon, 13 Jul 2020 12:00:00 -0000, which is more than 15 days"
    )

  def test_maximum_elapsed_time_no_total_seconds(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'https://www.erp5.com/',
        'status-code': '302',
        'ip-list': '127.0.0.1 127.0.0.2',
        'maximum-elapsed-time': '5',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.1",
            "status_code": 302,
            "url": "https://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.2",
            "status_code": 302,
            "url": "https://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "176.31.129.213",
            "status_code": 200,
            "url": "https://www.erp5.org/"
        }
    ],
    "ssl_certificate": [
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.1",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        },
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.2",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    self.launcher.run()
    self.assertPassedMessage(
      self.getPromiseResult(self.promise_name),
      "https://www.erp5.com/ : http_query: OK with status code 302 on IPs "
      "127.0.0.1 127.0.0.2 ssl_certificate: OK Certificate on 127.0.0.1 will "
      "expire on Mon, 13 Jul 2020 12:00:00 -0000, which is more than 15 days"
    )

  def test_http(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'http://www.erp5.com/',
        'status-code': '302',
        'ip-list': '127.0.0.1 127.0.0.2',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.1",
            "status_code": 302,
            "url": "http://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.2",
            "status_code": 302,
            "url": "http://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "176.31.129.213",
            "status_code": 200,
            "url": "http://www.erp5.org/"
        }
    ],
    "ssl_certificate": [
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    self.launcher.run()
    self.assertPassedMessage(
      self.getPromiseResult(self.promise_name),
      "http://www.erp5.com/ : http_query: OK with status code 302 on IPs 127.0.0.1 "
      "127.0.0.2"
    )

  def test_http_with_header_dict(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'http://www.erp5.com/',
        'status-code': '200',
        'http-header-dict': '{"Vary": "Accept-Encoding", "Cache-Control": "max-age=300, public"}',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "176.31.129.213",
            "http_header_dict": {"Vary": "Accept-Encoding", "Cache-Control": "max-age=300, public"},
            "status_code": 200,
            "url": "http://www.erp5.com/"
        }
    ],
    "ssl_certificate": [
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    self.launcher.run()
    self.assertPassedMessage(
      self.getPromiseResult(self.promise_name),
      "http://www.erp5.com/ : http_query: OK with status code 200 "
      "and HTTP Header dict {\"Cache-Control\": \"max-age=300, public\", \"Vary\": \"Accept-Encoding\"}"
    )

  def test_http_with_bad_header_dict(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'http://www.erp5.com/',
        'status-code': '200',
        'http-header-dict': '{"Vary": "Accept-Encoding", "Cache-Control": "max-age=300, public"}',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "176.31.129.213",
            "http_header_dict": {"Vary": "Accept-Encoding,Cookie", "Cache-Control": "max-age=300, public"},
            "status_code": 200,
            "url": "http://www.erp5.com/"
        }
    ],
    "ssl_certificate": [
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "http://www.erp5.com/ : http_query: ERROR HTTP Header dict was "
      "{\"Cache-Control\": \"max-age=300, public\", \"Vary\": \"Accept-Encoding,Cookie\"} "
      "instead of {\"Cache-Control\": \"max-age=300, public\", \"Vary\": \"Accept-Encoding\"}"
    )

  def test_no_ip_list(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'https://www.erp5.com/',
        'status-code': '302',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.1",
            "status_code": 302,
            "url": "https://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.2",
            "status_code": 302,
            "url": "https://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "176.31.129.213",
            "status_code": 200,
            "url": "https://www.erp5.org/"
        }
    ],
    "ssl_certificate": [
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.1",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        },
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.2",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    self.launcher.run()
    self.assertPassedMessage(
      self.getPromiseResult(self.promise_name),
      "https://www.erp5.com/ : http_query: OK with status code 302 "
      "ssl_certificate: OK Certificate on 127.0.0.1 will expire on Mon, 13 "
      "Jul 2020 12:00:00 -0000, which is more than 15 days"
    )

  def test_good_certificate_2_day(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'https://www.erp5.com/',
        'status-code': '302',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000',
        'certificate-expiration-days': '2'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.1",
            "status_code": 302,
            "url": "https://www.erp5.com/"
        }
    ],
    "ssl_certificate": [
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.1",
            "not_after": "Sun, 30 Dec 2019 12:00:00 -0000"
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    self.launcher.run()
    self.assertPassedMessage(
      self.getPromiseResult(self.promise_name),
      "https://www.erp5.com/ : http_query: OK with status code 302 "
      "ssl_certificate: OK Certificate on 127.0.0.1 will expire on Sun, 30 "
      "Dec 2019 12:00:00 -0000, which is more than 2 days"
    )

  def test_expired_certificate_2_day(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'https://www.erp5.com/',
        'status-code': '302',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000',
        'certificate-expiration-days': '2'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.1",
            "status_code": 302,
            "url": "https://www.erp5.com/"
        }
    ],
    "ssl_certificate": [
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.1",
            "not_after": "Sat, 28 Dec 2019 12:00:00 -0000"
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "https://www.erp5.com/ : ssl_certificate: ERROR Certificate on "
      "127.0.0.1 will expire on Sat, 28 Dec 2019 12:00:00 -0000, which is "
      "less than 2 days http_query: OK with status code 302"
    )

  def test_expired_certificate(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'https://www.erp5.com/',
        'status-code': '302',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.1",
            "status_code": 302,
            "url": "https://www.erp5.com/"
        }
    ],
    "ssl_certificate": [
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.1",
            "not_after": "Sat, 28 Dec 2019 12:00:00 -0000"
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "https://www.erp5.com/ : ssl_certificate: ERROR Certificate on "
      "127.0.0.1 will expire on Sat, 28 Dec 2019 12:00:00 -0000, which is "
      "less than 15 days http_query: OK with status code 302"
    )

  def test_expired_certificate_before_today(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'https://www.erp5.com/',
        'status-code': '302',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.1",
            "status_code": 302,
            "url": "https://www.erp5.com/"
        }
    ],
    "ssl_certificate": [
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.1",
            "not_after": "Thu, 26 Dec 2019 12:00:00 -0000"
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "https://www.erp5.com/ : ssl_certificate: ERROR Certificate on "
      "127.0.0.1 will expire on Thu, 26 Dec 2019 12:00:00 -0000, which is "
      "less than 15 days http_query: OK with status code 302"
    )

  def test_no_http_query_data(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'https://www.erp5.com/',
        'status-code': '302',
        'ip-list': '127.0.0.1 127.0.0.2',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
    ],
    "ssl_certificate": [
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.1",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        },
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.2",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "https://www.erp5.com/ : http_query: ERROR No data elapsed_time: ERROR "
      "No data ssl_certificate: OK Certificate on 127.0.0.1 will expire on "
      "Mon, 13 Jul 2020 12:00:00 -0000, which is more than 15 days"
    )

  def test_no_ssl_certificate_data(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'https://www.erp5.com/',
        'status-code': '302',
        'ip-list': '127.0.0.1 127.0.0.2',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.1",
            "status_code": 302,
            "url": "https://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.2",
            "status_code": 302,
            "url": "https://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "176.31.129.213",
            "status_code": 200,
            "url": "https://www.erp5.org/"
        }
    ],
    "ssl_certificate": [
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "https://www.erp5.com/ : ssl_certificate: ERROR No data http_query: "
      "OK with status code 302 on IPs 127.0.0.1 127.0.0.2"
    )

  def test_no_ssl_certificate(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'https://www.erp5.com/',
        'status-code': '302',
        'ip-list': '127.0.0.1 127.0.0.2',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.1",
            "status_code": 302,
            "url": "https://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.2",
            "status_code": 302,
            "url": "https://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "176.31.129.213",
            "status_code": 200,
            "url": "https://www.erp5.org/"
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "https://www.erp5.com/ : ssl_certificate: ERROR No key "
      "'ssl_certificate'. If the error persist, please update surykatka. "
      "http_query: OK with status code 302 on IPs 127.0.0.1 127.0.0.2"
    )

  def test_bad_code(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'https://www.erp5.com/',
        'status-code': '301',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.1",
            "status_code": 302,
            "url": "https://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.2",
            "status_code": 301,
            "url": "https://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "176.31.129.213",
            "status_code": 200,
            "url": "https://www.erp5.org/"
        }
    ],
    "ssl_certificate": [
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.1",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        },
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.2",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "https://www.erp5.com/ : http_query: ERROR IP 127.0.0.1 got status code "
      "302 instead of 301 ssl_certificate: OK Certificate on 127.0.0.1 will "
      "expire on Mon, 13 Jul 2020 12:00:00 -0000, which is more than 15 days"
    )

  def _test_bad_code_explanation(self, status_code, explanation):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'https://www.erp5.com/',
        'status-code': '301',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.1",
            "status_code": %s,
            "url": "https://www.erp5.com/"
        }
    ],
    "ssl_certificate": [
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.1",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        },
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.2",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        }
    ]
}
""" % status_code)
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "https://www.erp5.com/ : http_query: ERROR IP 127.0.0.1 got status code "
      "%s instead of 301 ssl_certificate: OK Certificate on 127.0.0.1 will "
      "expire on Mon, 13 Jul 2020 12:00:00 -0000, which is more than 15 days"
      "" % (explanation,)
    )

  def test_bad_code_explanation_520(self):
    self._test_bad_code_explanation(520, '520 (Too many redirects)')

  def test_bad_code_explanation_523(self):
    self._test_bad_code_explanation(523, '523 (Connection error)')

  def test_bad_code_explanation_524(self):
    self._test_bad_code_explanation(524, '524 (Connection timeout)')

  def test_bad_code_explanation_526(self):
    self._test_bad_code_explanation(526, '526 (SSL Error)')

  def test_bad_ip(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'https://www.erp5.com/',
        'status-code': '301',
        'ip-list': '127.0.0.1 127.0.0.2',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.1",
            "status_code": 301,
            "url": "https://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.4",
            "status_code": 301,
            "url": "https://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "176.31.129.213",
            "status_code": 200,
            "url": "https://www.erp5.org/"
        }
    ],
    "ssl_certificate": [
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.1",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        },
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.2",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "https://www.erp5.com/ : http_query: ERROR expected IPs 127.0.0.1 "
      "127.0.0.2 differes from got 127.0.0.1 127.0.0.4 ssl_certificate: "
      "OK Certificate on 127.0.0.1 will expire on Mon, 13 Jul 2020 12:00:00 "
      "-0000, which is more than 15 days"
    )

  def test_bad_ip_status_code(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'https://www.erp5.com/',
        'status-code': '301',
        'ip-list': '127.0.0.1 127.0.0.2',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.1",
            "status_code": 302,
            "url": "https://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.4",
            "status_code": 301,
            "url": "https://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "176.31.129.213",
            "status_code": 200,
            "url": "https://www.erp5.org/"
        }
    ],
    "ssl_certificate": [
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.1",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        },
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.2",
            "not_after": "Mon, 13 Jul 2020 12:00:00 -0000"
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "https://www.erp5.com/ : http_query: ERROR IP 127.0.0.1 got status code "
      "302 instead of 301 http_query: ERROR expected IPs 127.0.0.1 127.0.0.2 "
      "differes from got 127.0.0.1 127.0.0.4 ssl_certificate: OK Certificate "
      "on 127.0.0.1 will expire on Mon, 13 Jul 2020 12:00:00 -0000, which is "
      "more than 15 days"
    )

  def test_https_no_cert(self):
    self.writeSurykatkaPromise(
      {
        'report': 'http_query',
        'json-file': self.json_file,
        'url': 'https://www.erp5.com/',
        'status-code': '301',
        'ip-list': '127.0.0.1 127.0.0.2',
        'test-utcnow': 'Fri, 27 Dec 2019 15:11:12 -0000'
      }
    )
    self.writeSurykatkaJson("""{
    "http_query": [
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.1",
            "status_code": 302,
            "url": "https://www.erp5.com/"
        },
        {
            "date": "Wed, 11 Dec 2019 09:35:28 -0000",
            "ip": "127.0.0.4",
            "status_code": 301,
            "url": "https://www.erp5.com/"
        }
    ],
    "ssl_certificate": [
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.1",
            "not_after": null
        },
        {
            "date": "Fri, 27 Dec 2019 14:43:26 -0000",
            "hostname": "www.erp5.com",
            "ip": "127.0.0.2",
            "not_after": null
        }
    ]
}
""")
    self.configureLauncher(enable_anomaly=True)
    with self.assertRaises(PromiseError):
      self.launcher.run()
    self.assertFailedMessage(
      self.getPromiseResult(self.promise_name),
      "https://www.erp5.com/ : http_query: ERROR IP 127.0.0.1 got status code "
      "302 instead of 301 http_query: ERROR expected IPs 127.0.0.1 127.0.0.2 "
      "differes from got 127.0.0.1 127.0.0.4 ssl_certificate: ERROR No "
      "certificate information for 127.0.0.1 ssl_certificate: ERROR No "
      "certificate information for 127.0.0.2"
    )



# -*- coding: utf-8 -*-
##############################################################################
# Copyright (c) 2018 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly advised to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
##############################################################################

import os
import socket
import time
from slapos.grid.promise import PromiseError
from slapos.promise.plugin.check_socket_listening import RunPromise
from . import TestPromisePluginMixin


SLAPOS_TEST_IPV4 = os.environ.get('SLAPOS_TEST_IPV4', '127.0.0.1')
SLAPOS_TEST_IPV6 = os.environ.get('SLAPOS_TEST_IPV6', '::1')


class TestCheckSocketListening(TestPromisePluginMixin):

  promise_name = "check-socket-listening.py"
  promise_template = "from %s import %s\nextra_config_dict = %%r\n" % (
    RunPromise.__module__, RunPromise.__name__)

  def writePromise(self, **kw):
    super(TestCheckSocketListening, self).writePromise(
      self.promise_name, self.promise_template % kw)

  def test_input_conflict(self):
    for keys in (
        (), ('port',),
        ('host',), ('host', 'pathname'), ('host', 'abstract'),
        ('host', 'abstract', 'pathname'), ('pathname', 'abstract')):
      self.writePromise(**dict.fromkeys(keys, 'a'))
      self.configureLauncher(force=True)
      self.assertRaises(PromiseError, self.launcher.run)

  def test_host_port(self):
    for host, family in ((SLAPOS_TEST_IPV4, socket.AF_INET),
                         (SLAPOS_TEST_IPV6, socket.AF_INET6)):
      s = socket.socket(family)
      try:
        s.bind((host, 0))
        port = s.getsockname()[1]
        self.writePromise(host=host, port=str(port))
        self.configureLauncher(force=True)
        self.assertRaises(PromiseError, self.launcher.run)
        s.listen(5)
        time.sleep(1)
        self.launcher.run()
      finally:
        s.close()

  def test_unix_socket(self):
    name = os.path.join(self.partition_dir, 'test_unix_socket')
    for addr, abstract in ((name, False), (name, True)):
      s = socket.socket(socket.AF_UNIX)
      try:
        if abstract:
          s.bind('\0' + addr)
          self.writePromise(abstract=addr)
        else:
          s.bind(addr)
          self.writePromise(pathname=addr)
        self.configureLauncher(force=True)
        self.assertRaises(PromiseError, self.launcher.run)
        s.listen(5)
        time.sleep(1)
        self.launcher.run()
      finally:
        s.close()



import sys
import pycurl
from io import BytesIO
from slapos.util import bytes2str

def get_curl(buffer, url):
  curl = pycurl.Curl()
  curl.setopt(curl.URL, url)
  curl.setopt(curl.CONNECTTIMEOUT, 10)
  curl.setopt(curl.TIMEOUT, 30)
  curl.setopt(curl.WRITEDATA, buffer)
  curl.setopt(curl.SSL_VERIFYPEER, False)
  curl.setopt(curl.SSL_VERIFYHOST, False)
  result = "OK"
  try:
    curl.perform()
  except Exception:
    import traceback
    traceback.print_exc(file=sys.stderr)
    sys.stderr.flush()
    result = "FAIL"
  return curl, result

def request(url, expected_dict):
  
  buffer = BytesIO()
  curl, result = get_curl(buffer, url)

  body = buffer.getvalue()
  
  rendering_time = "%s;%s;%s;%s;%s" % \
    (curl.getinfo(curl.NAMELOOKUP_TIME),
     curl.getinfo(curl.CONNECT_TIME),
     curl.getinfo(curl.PRETRANSFER_TIME),
     curl.getinfo(curl.STARTTRANSFER_TIME),
     curl.getinfo(curl.TOTAL_TIME))
  
  response_code = curl.getinfo(pycurl.HTTP_CODE)

  expected_response = expected_dict.get("expected_response", None)
  if expected_response is not None and \
      expected_response != response_code:
    result = "UNEXPECTED (%s != %s)" % (expected_response, response_code)

  expected_text = expected_dict.get("expected_text", None)
  if expected_text is not None and \
      str(expected_text) not in bytes2str(body):
    result = "UNEXPECTED (%s not in page content)" % (expected_text)

  curl.close()

  info_list = ('GET', url, response_code, rendering_time, result)

  return info_list




# -*- coding: utf-8 -*-
# vim: set et sts=2:
# pylint: disable-msg=W0311,C0301,C0103,C0111

import datetime
import hashlib
import os
import re
import shutil
from six.moves.urllib.parse import unquote
import zipfile
import fnmatch

import werkzeug
from slapos.runner.utils import realpath, tail, isText
from slapos.util import str2bytes


class FileBrowser(object):
  """This class contains all base functions for file browser"""

  def __init__(self, config):
    self.config = config

  def _realdir(self, dir):
    realdir = realpath(self.config, unquote(dir))
    if not realdir:
      raise NameError('Could not load directory %s: Permission denied' % dir)
    return realdir

  def _filterPathList(self, path_list):
    """Filter out paths that matches a list ignored file patterns.
    """
    # This could be configurable ...
    ignored_file_list = '''
*.py[cod]
*.o
'''
    for pattern in ignored_file_list.splitlines():
      if pattern.strip():
        path_list = [path for path in path_list if not fnmatch.fnmatch(path, pattern)]
    return path_list


  def listDirs(self, dir, all=False):
    """List elements of directory 'dir' taken"""
    html = 'var gsdirs = [], gsfiles = [];'

    dir = unquote(dir)
    # XXX-Marco 'dir' and 'all' should not shadow builtin names
    realdir = realpath(self.config, dir)
    if not realdir:
      raise NameError('Could not load directory %s: Permission denied' % dir)

    ldir = sorted(os.listdir(realdir), key=str.lower)
    for f in self._filterPathList(ldir):
      if f.startswith('.') and not all:  # do not display this file/folder
        continue
      ff = os.path.join(dir, f)
      realfile = os.path.join(realdir, f)
      mdate = datetime.datetime.fromtimestamp(os.path.getmtime(realfile)
                                              ).strftime("%Y-%d-%m %I:%M")
      md5sum = hashlib.md5(str2bytes(realfile)).hexdigest()
      if not os.path.isdir(realfile):
        size = os.path.getsize(realfile)
        regex = re.compile("(^.*)\.(.*)", re.VERBOSE)
        ext = regex.sub(r'\2', f)
        if ext == f:
          ext = "unknow"
        else:
          ext = str.lower(ext)
        html += 'gsfiles.push(new gsItem("1", "%s", "%s", "%s", "%s", "%s", "%s"));' % (f, ff, size, md5sum, ext, mdate)
      else:
        html += 'gsdirs.push(new gsItem("2", "%s", "%s", "0", "%s", "dir", "%s"));' % (f, ff, md5sum, mdate)
    return html

  def fancylistDirs(self, dir, key, listfiles, all=False):
    dir = unquote(dir)
    realdir = realpath(self.config, dir)
    if not realdir:
      raise NameError('Could not load directory %s: Permission denied' % dir)
    fileList = []
    dirList = []
    i = 0
    ldir = sorted(os.listdir(realdir), key=str.lower)
    for f in self._filterPathList(ldir):
      if f.startswith('.') and not all:  # do not display this file/folder
        continue
      ff = os.path.join(dir, f)
      realfile = os.path.join(realdir, f)
      identity = "%s%s" % (key, i)
      if os.path.isdir(realfile):
        dirList.append({"title": f, "key": identity,
                        "folder":True, "lazy":True, "path": ff})
      elif listfiles:
        regex = re.compile("(^.*)\.(.*)", re.VERBOSE)
        ext = regex.sub(r'\2', f)
        if ext == f:
          ext = ""
        fileList.append({"title": f, "key": identity,
                        "extraClasses": "ext_"+ext, "path": ff})
      i+=1
    return (dirList + fileList)

  def makeDirectory(self, dir, filename):
    """Create a directory"""
    realdir = self._realdir(dir)
    folder = os.path.join(realdir, filename)
    if not os.path.exists(folder):
      os.mkdir(folder, 0o744)
      return "{result: '1'}"
    else:
      return "{result: '0'}"

  def makeFile(self, dir, filename):
    """Create a file in a directory dir taken"""
    realdir = self._realdir(dir)
    fout = os.path.join(realdir, filename)
    if not os.path.exists(fout):
      open(fout, 'w')
      return "var responce = {result: '1'}"
    else:
      return "{result: '0'}"

  def deleteItem(self, dir, files):
    """Delete a list of files or directories"""
    # XXX-Marco do not shadow 'dir'
    realdir = self._realdir(dir)
    lfiles = unquote(files).split(',,,')
    try:
      # XXX-Marco do not shadow 'file'
      for item in lfiles:
        filepath = os.path.join(realdir, item)
        if not item or not os.path.exists(filepath):
          continue  # silent skip file....
        details = filepath.split('/')
        last = details[-1]
        if last and last.startswith('.'):
          continue  # cannot delete this file/directory, to prevent security
        if os.path.isdir(filepath):
          shutil.rmtree(filepath)
        else:
          os.unlink(filepath)
    except Exception as e:
      return str(e)
    return "{result: '1'}"

  def copyItem(self, dir, files, del_source=False):
    """Copy a list of files or directory to dir"""
    realdir = self._realdir(dir)
    lfiles = unquote(files).split(',,,')
    try:
      # XXX-Marco do not shadow 'file'
      for file in lfiles:
        realfile = realpath(self.config, file)
        if not realfile:
          raise NameError('Could not load file or directory %s: Permission denied' % file)
        #prepare destination file
        details = realfile.split('/')
        dest = os.path.join(realdir, details[-1])
        if os.path.exists(dest):
          raise NameError('NOT ALLOWED OPERATION : File or directory already exists')
        if os.path.isdir(realfile):
          shutil.copytree(realfile, dest)
          if del_source:
            shutil.rmtree(realfile)
        else:
          shutil.copy(realfile, dest)
          if del_source:
            os.unlink(realfile)
    except Exception as e:
      return str(e)
    return "{result: '1'}"

  def rename(self, dir, filename, newfilename):
    """Rename file or directory to dir/filename"""
    realdir = self._realdir(dir)
    realfile = realpath(self.config, unquote(filename))
    if not realfile:
      raise NameError('Could not load directory %s: Permission denied' % filename)
    tofile = os.path.join(realdir, newfilename)
    if not os.path.exists(tofile):
      os.rename(realfile, tofile)
      return "{result: '1'}"
    raise NameError('NOT ALLOWED OPERATION : File or directory already exists')

  def copyAsFile(self, dir, filename, newfilename):
    """Copy file or directory to dir/filename"""
    realdir = self._realdir(dir)
    fromfile = os.path.join(realdir, filename)
    tofile = os.path.join(realdir, newfilename)
    if not os.path.exists(fromfile):
      raise NameError('NOT ALLOWED OPERATION : File or directory does not exist')
    while os.path.exists(tofile):
      tofile += "_1"
    shutil.copy(fromfile, tofile)
    return "{result: '1'}"

  def uploadFile(self, dir, files):
    """Upload a list of files in directory dir"""
    realdir = self._realdir(dir)
    for file in files:
      if files[file]:
        filename = werkzeug.secure_filename(files[file].filename)
        if not os.path.exists(os.path.join(dir, filename)):
          files[file].save(os.path.join(realdir, filename))
    return "{result: '1'}"

  def downloadFile(self, dir, filename):
    """Download file dir/filename"""
    realdir = self._realdir(dir)
    file = os.path.join(realdir, unquote(filename))
    if not os.path.exists(file):
      raise NameError('NOT ALLOWED OPERATION : File or directory does not exist %s'
                      % os.path.join(dir, filename))
    return file

  def zipFile(self, dir, filename, newfilename):
    """Add filename to archive as newfilename"""
    realdir = self._realdir(dir)
    tozip = os.path.join(realdir, newfilename)
    fromzip = os.path.join(realdir, filename)
    if not os.path.exists(fromzip):
      raise NameError('NOT ALLOWED OPERATION : File or directory does not exist')
    if not os.path.exists(tozip):
      zip = zipfile.ZipFile(tozip, 'w', zipfile.ZIP_DEFLATED)
      if os.path.isdir(fromzip):
        rootlen = len(fromzip) + 1
        for base, _, files in os.walk(fromzip):
          for filename in files:
            fn = os.path.join(base, filename).encode("utf-8")
            zip.write(fn, fn[rootlen:])       # XXX can fail if 'fromzip' contains multibyte characters
      else:
        zip.write(fromzip)
      zip.close()
      return "{result: '1'}"
    raise NameError('NOT ALLOWED OPERATION : File or directory already exists')

  def unzipFile(self, dir, filename, newfilename):
    """Extract a zipped archive"""
    realdir = self._realdir(dir)
    target = os.path.join(realdir, newfilename)
    archive = os.path.join(realdir, filename)
    if not os.path.exists(archive):
      raise NameError('NOT ALLOWED OPERATION : File or directory does not exist')
    if not os.path.exists(target):
      zip = zipfile.ZipFile(archive)
      #member = zip.namelist()
      zip.extractall(target)
      #if len(member) > 1:
      #  zip.extractall(target)
      #else:
      #  zip.extract(member[0], newfilename)
      return "{result: '1'}"
    raise NameError('NOT ALLOWED OPERATION : File or directory already exists')

  def readFile(self, dir, filename, truncate=False):
    """Read file dir/filename and return content"""
    realfile = realpath(self.config, os.path.join(unquote(dir),
                        unquote(filename)))
    if not realfile:
      raise NameError('Could not load directory %s: Permission denied' % dir)
    if not isText(realfile):
      return "FILE ERROR: Cannot display binary file, please open a text file only!"
    if not truncate:
      return open(realfile).read()
    else:
      return tail(open(realfile), 0)



##############################################################################
#
# Copyright (c) 2017 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################
from __future__ import unicode_literals
import unittest
import os
import time
import tempfile
import datetime
import shutil
import codecs
from backports import lzma

from . import data
from slapos.promise.check_slow_queries_digest_result import checkMariadbDigestResult

class TestCheckSlowQueriesDigestResult(unittest.TestCase):

  base_path, = data.__path__

  def _create_file(self, date, with_content):
    content = ''
    if with_content:
      with codecs.open(os.path.join(self.base_path, "ptdigest.txt"), encoding='utf-8') as f:
        content = f.read()

    name = date.strftime('slowquery_digest.txt-%Y-%m-%d.xz')
    oldtime = time.mktime(date.timetuple()) + 2000
    with lzma.open( self.base_dir+name, 'at') as the_file:
      the_file.write(content)
    os.utime(self.base_dir+name, ( oldtime , oldtime ))
    
  def _remove_file(self, date):
    name = date.strftime('slowquery_digest.txt-%Y-%m-%d.xz')
    os.remove(self.base_dir+name)

  def setUp(self):
    self.base_dir = "/tmp/ap/"
    if not os.path.exists(self.base_dir):
      os.makedirs(self.base_dir)
      os.utime(self.base_dir, (time.time() - 202800, time.time() - 202800))

    # create test files
    self.today = datetime.date.today()
    self._create_file(self.today, True)

    self.yesterday = (self.today - datetime.timedelta(1))
    self._create_file(self.yesterday, False)
    _, self.status_file = tempfile.mkstemp()

  def test_threshold_is_greater(self):
    status, message = checkMariadbDigestResult(self.base_dir, self.status_file, 5000, 100)
    self.assertEqual("Thanks for keeping it all clean, total queries are : 3420.0 slowest query time is : 34", message)
    self.assertEqual(0, status)

  def test_no_today_file_and_empty_yesterday_file(self):
    self._remove_file(self.today)
    status, message = checkMariadbDigestResult(self.base_dir, self.status_file, 60, 100)
    self.assertEqual("No result found in the slow query digest file or the file is corrupted", message)
    self.assertEqual(1, status)
    self._create_file(self.today, True)

  def test_fail(self):
    status, message = checkMariadbDigestResult(self.base_dir, self.status_file, 90, 100)
    self.assertEqual("Threshold is lower than expected: \nExpected total queries : 90 and current is: 3420.0\nExpected slowest query : 100 and current is: 34", message)
    self.assertEqual(1, status)

  def test_no_today_file_but_yesterday_file(self):
    self._remove_file(self.today)
    self._create_file(self.yesterday, True)
    status, message = checkMariadbDigestResult(self.base_dir, self.status_file, 5000, 100)
    self.assertEqual("Thanks for keeping it all clean, total queries are : 3420.0 slowest query time is : 34", message)
    self.assertEqual(0, status)
    self._create_file(self.today, True)
    self._remove_file(self.yesterday)
    self._create_file(self.yesterday, False)
    
  def tearDown(self):
    self._remove_file(self.today)
    self._remove_file(self.yesterday)
    shutil.rmtree('/tmp/ap')

if __name__ == '__main__':
  unittest.main()




##############################################################################
#
# Copyright (c) 2017 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import unittest
import os.path
import socket
from slapos.promise.is_local_tcp_port_opened import isLocalTcpPortOpened


class TestLocalTcpPortOpened(unittest.TestCase):

  def test_port_is_not_open(self):
    self.assertEqual(isLocalTcpPortOpened("127.0.0.1",65550), False)

  def test_port6_is_not_open(self):
    self.assertEqual(isLocalTcpPortOpened("::1",65550), False)
 
  def test_port_is_open(self):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
      s.bind(("127.0.0.1", 0))
      s.listen(1)
      port = s.getsockname()[1]
      self.assertEqual(isLocalTcpPortOpened("127.0.0.1",port), True)
    finally:
      s.close()

  def test_port6_is_open(self):
    s = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
    try:
      s.bind(("::1", 0))
      s.listen(1)
      port = s.getsockname()[1]
      self.assertEqual(isLocalTcpPortOpened("::1",port), True)
    finally:
      s.close()

    
  
if __name__ == '__main__':
  unittest.main()



from __future__ import division

from zope.interface import implementer
from slapos.grid.promise import interface
from slapos.grid.promise.generic import GenericPromise

import os
import sys

import sqlite3
import argparse
import datetime
import psutil

from slapos.collect.db import Database

@implementer(interface.IPromise)
class RunPromise(GenericPromise):

  def __init__(self, config):
    super(RunPromise, self).__init__(config)
    # check disk space at least every 3 minutes
    self.setPeriodicity(minute=3)



  def getDiskSize(self, disk_partition, database):
    database = Database(database, create=False, timeout=10)
    try:
      # fetch disk size
      database.connect()
      where_query = "partition='%s'" % (disk_partition)
      order = "datetime(date || ' ' || time) DESC"
      query_result = database.select("disk", columns="free+used", where=where_query, order=order, limit=1)
      result = query_result.fetchone()
      if not result or not result[0]:
        return None
      disk_size = result[0]
    except sqlite3.OperationalError as e:
      # if database is still locked after timeout expiration (another process is using it)
      # we print warning message and try the promise at next run until max warn count
      locked_message = "database is locked"
      if locked_message in str(e) and \
          not self.raiseOnDatabaseLocked(locked_message):
        return None
      raise
    finally:
      try:
        database.close()
      except Exception:
        pass
    return disk_size

  def getFreeSpace(self, disk_partition, database, date, time):
    database = Database(database, create=False, timeout=10)
    try:
      # fetch free disk space
      database.connect()
      where_query = "time between '%s:00' and '%s:30' and partition='%s'" % (time, time, disk_partition)
      query_result = database.select("disk", date, "free", where=where_query)
      result = query_result.fetchone()
      if not result or not result[0]:
        self.logger.info("No result from collector database: disk check skipped")
        return 0
      disk_free = result[0]
    except sqlite3.OperationalError as e:
      # if database is still locked after timeout expiration (another process is using it)
      # we print warning message and try the promise at next run until max warn count
      locked_message = "database is locked"
      if locked_message in str(e) and \
          not self.raiseOnDatabaseLocked(locked_message):
        return 0
      raise
    finally:
      try:
        database.close()
      except Exception:
        pass
    return int(disk_free)

  def raiseOnDatabaseLocked(self, locked_message):
    max_warn = 10
    latest_result_list = self.getLastPromiseResultList(result_count=max_warn)
    warning_count = 0
    if len(latest_result_list) < max_warn:
      return False

    for result in latest_result_list[0]:
      if result['status'] == "ERROR" and locked_message in result["message"]:
        return True

    for result_list in latest_result_list:
      found = False
      for result in result_list:
        if result['status'] == "WARNING" and locked_message in result["message"]:
          found = True
          warning_count += 1
          break
      if not found:
        break
    if warning_count == max_warn:
      # too many warning on database locked, now fail.
      return True

    self.logger.warn("collector database is locked by another process")
    return False

  @staticmethod
  def _checkInodeUsage(path):
    stat = os.statvfs(path)
    total_inode = stat.f_files
    if total_inode:
      usage = 100 * (total_inode - stat.f_ffree) / total_inode
      if usage >= 98:
        return "Disk Inodes usage is really high: %.4f%%" % usage

  def getInodeUsage(self, path):
    return (self._checkInodeUsage(path) or
       os.path.ismount('/tmp') and self._checkInodeUsage('/tmp') or
       "")

  def sense(self):
    # find if a disk is mounted on the path
    disk_partition = ""
    db_path = self.getConfig('collectordb')
    check_date = self.getConfig('test-check-date')
    path = os.path.join(self.getPartitionFolder(), "") + "extrafolder"
    partitions = psutil.disk_partitions()
    while path is not '/':
      if not disk_partition:
        path = os.path.dirname(path)
      else:
        break
      for p in partitions:
        if p.mountpoint == path:
          disk_partition = p.device
          break
    if not disk_partition:
      self.logger.error("Couldn't find disk partition")
      return

    if db_path.endswith("collector.db"):
      db_path=db_path[:-len("collector.db")]

    if check_date:
      # testing mode
      currentdate = check_date
      currenttime = self.getConfig('test-check-time', '09:17')
      disk_partition = self.getConfig('test-disk-partition', '/dev/sda1')
    else:
      # get last minute
      now = datetime.datetime.now()
      currentdate = now.strftime('%Y-%m-%d')
      currenttime = now - datetime.timedelta(minutes=1)
      currenttime = currenttime.time().strftime('%H:%M')

    disk_size = self.getDiskSize(disk_partition, db_path)
    default_threshold = None
    if disk_size is not None:
      default_threshold = round(disk_size/(1024*1024*1024) * 0.05, 2)
    threshold = float(self.getConfig('threshold', default_threshold) or 2.0)
    threshold_days = float(self.getConfig('threshold-days', '30'))

    free_space = self.getFreeSpace(disk_partition, db_path, currentdate,
                                   currenttime)
    if free_space == 0:
      return
    elif free_space > threshold*1024*1024*1024:
      inode_usage = self.getInodeUsage(self.getPartitionFolder())
      if inode_usage:
        self.logger.error(inode_usage)
      else:
        self.logger.info("Disk usage: OK")
      return

    free_space = round(free_space/(1024*1024*1024), 2)
    self.logger.error('Free disk space low: remaining %s G (threshold: %s G)' % (
      free_space, threshold))

  def test(self):
    return self._test(result_count=1, failure_amount=1)

  def anomaly(self):
    return self._test(result_count=3, failure_amount=3)



from zope.interface import implementer
from slapos.grid.promise import interface
from slapos.grid.promise.generic import GenericPromise
import os
from datetime import datetime

@implementer(interface.IPromise)
class RunPromise(GenericPromise):

  def __init__(self, config):
    super(RunPromise, self).__init__(config)
    self.setPeriodicity(minute=1)

  def sense(self):
    """
      Run the promise code and store the result
        raise error, log error message, ... for failure
    """

    partition_folder = self.getPartitionFolder()
    log_folder = os.path.join(partition_folder, 'var/log')
    log_name = 'slapgrid-%s-error.log' % self.getConfig('partition-id')
    slapgrid_error_log_file = os.path.join(partition_folder, '.%s' % log_name)
    link_file = os.path.join(log_folder, log_name)
    monitor_url = self.getConfig('monitor-url')
    message = ''
    if os.path.exists(slapgrid_error_log_file) and \
        os.stat(slapgrid_error_log_file).st_size:
      message = 'Buildout failed to process %s.' % self.getConfig('partition-id')
      if monitor_url:
        message += '\nSee %s/log/%s for more information.' % (monitor_url, log_name)
      if not os.path.exists(link_file):
        os.symlink(slapgrid_error_log_file, link_file)
    else:
      if os.path.exists(link_file):
        os.unlink(link_file)

    if message:
      self.logger.error(message)
    else:
      self.logger.info("buildout is OK")

  def test(self):
    """
      Test promise and say if problem is detected or not
      Return TestResult object
    """
    return self._test(result_count=1, failure_amount=1)

  def anomaly(self):
    return self._test(result_count=2, failure_amount=2)
